@@ -2,12 +2,11 @@
 from __future__ import absolute_import
 
 import base64
-import codecs
 import configparser
-import csv
+import logging
 import os
 from datetime import datetime, timedelta
-from typing import Iterator, List, Generator, Any, Union, Iterable, AsyncGenerator
+from typing import List, Generator, Any, Union, Iterable, AsyncGenerator
 
 from urllib3 import HTTPResponse
 
@@ -16,9 +15,10 @@ from influxdb_client import Configuration, Dialect, Query, OptionStatement, Vari
     Duration, StringLiteral, ArrayExpression, ImportDeclaration, MemberExpression, MemberAssignment, File, \
     WriteService, QueryService, DeleteService, DeletePredicateRequest
 from influxdb_client.client.flux_csv_parser import FluxResponseMetadataMode, FluxCsvParser, FluxSerializationMode
-from influxdb_client.client.flux_table import FluxTable, FluxRecord
+from influxdb_client.client.flux_table import FluxRecord, TableList, CSVIterator
 from influxdb_client.client.util.date_utils import get_date_helper
 from influxdb_client.client.util.helpers import get_org_query_param
+from influxdb_client.client.warnings import MissingPivotFunction
 from influxdb_client.client.write.dataframe_serializer import DataframeSerializer
 from influxdb_client.rest import _UTF_8_encoding
 
@@ -29,11 +29,23 @@ try:
 except ModuleNotFoundError:
     _HAS_DATACLASS = False
 
+LOGGERS_NAMES = [
+    'influxdb_client.client.influxdb_client',
+    'influxdb_client.client.influxdb_client_async',
+    'influxdb_client.client.write_api',
+    'influxdb_client.client.write_api_async',
+    'influxdb_client.client.write.retry',
+    'influxdb_client.client.write.dataframe_serializer',
+    'influxdb_client.client.util.multiprocessing_helper',
+    'influxdb_client.client.http',
+    'influxdb_client.client.exceptions',
+]
+
 
 # noinspection PyMethodMayBeStatic
 class _BaseClient(object):
     def __init__(self, url, token, debug=None, timeout=10_000, enable_gzip=False, org: str = None,
-                 default_tags: dict = None, **kwargs) -> None:
+                 default_tags: dict = None, http_client_logger: str = None, **kwargs) -> None:
         self.url = url
         self.token = token
         self.org = org
@@ -46,21 +58,40 @@ class _BaseClient(object):
         else:
             self.conf.host = self.url
         self.conf.enable_gzip = enable_gzip
-        self.conf.debug = debug
         self.conf.verify_ssl = kwargs.get('verify_ssl', True)
         self.conf.ssl_ca_cert = kwargs.get('ssl_ca_cert', None)
+        self.conf.cert_file = kwargs.get('cert_file', None)
+        self.conf.cert_key_file = kwargs.get('cert_key_file', None)
+        self.conf.cert_key_password = kwargs.get('cert_key_password', None)
+        self.conf.ssl_context = kwargs.get('ssl_context', None)
         self.conf.proxy = kwargs.get('proxy', None)
         self.conf.proxy_headers = kwargs.get('proxy_headers', None)
         self.conf.connection_pool_maxsize = kwargs.get('connection_pool_maxsize', self.conf.connection_pool_maxsize)
         self.conf.timeout = timeout
+        # logging
+        self.conf.loggers["http_client_logger"] = logging.getLogger(http_client_logger)
+        for client_logger in LOGGERS_NAMES:
+            self.conf.loggers[client_logger] = logging.getLogger(client_logger)
+        self.conf.debug = debug
 
-        auth_token = self.token
-        self.auth_header_name = "Authorization"
-        self.auth_header_value = "Token " + auth_token
-
+        self.conf.username = kwargs.get('username', None)
+        self.conf.password = kwargs.get('password', None)
+        # defaults
+        self.auth_header_name = None
+        self.auth_header_value = None
+        # by token
+        if self.token:
+            self.auth_header_name = "Authorization"
+            self.auth_header_value = "Token " + self.token
+        # by HTTP basic
         auth_basic = kwargs.get('auth_basic', False)
         if auth_basic:
+            self.auth_header_name = "Authorization"
             self.auth_header_value = "Basic " + base64.b64encode(token.encode()).decode()
+        # by username, password
+        if self.conf.username and self.conf.password:
+            self.auth_header_name = None
+            self.auth_header_value = None
 
         self.retries = kwargs.get('retries', False)
 
@@ -75,66 +106,99 @@ class _BaseClient(object):
         return "unknown"
 
     @classmethod
-    def _from_config_file(cls, config_file: str = "config.ini", debug=None, enable_gzip=False):
+    def _from_config_file(cls, config_file: str = "config.ini", debug=None, enable_gzip=False, **kwargs):
         config = configparser.ConfigParser()
-        config.read(config_file)
+        is_json = False
+        try:
+            config.read(config_file)
+        except configparser.ParsingError:
+            with open(config_file) as json_file:
+                import json
+                config = json.load(json_file)
+                is_json = True
 
-        def config_value(key: str):
-            return config['influx2'][key].strip('"')
+        def _config_value(key: str):
+            value = str(config[key]) if is_json else config['influx2'][key]
+            return value.strip('"')
 
-        url = config_value('url')
-        token = config_value('token')
+        def _has_option(key: str):
+            return key in config if is_json else config.has_option('influx2', key)
+
+        def _has_section(key: str):
+            return key in config if is_json else config.has_section(key)
+
+        url = _config_value('url')
+        token = _config_value('token')
 
         timeout = None
-        if config.has_option('influx2', 'timeout'):
-            timeout = config_value('timeout')
+        if _has_option('timeout'):
+            timeout = _config_value('timeout')
 
         org = None
-        if config.has_option('influx2', 'org'):
-            org = config_value('org')
+        if _has_option('org'):
+            org = _config_value('org')
 
         verify_ssl = True
-        if config.has_option('influx2', 'verify_ssl'):
-            verify_ssl = config_value('verify_ssl')
+        if _has_option('verify_ssl'):
+            verify_ssl = _config_value('verify_ssl')
 
         ssl_ca_cert = None
-        if config.has_option('influx2', 'ssl_ca_cert'):
-            ssl_ca_cert = config_value('ssl_ca_cert')
+        if _has_option('ssl_ca_cert'):
+            ssl_ca_cert = _config_value('ssl_ca_cert')
+
+        cert_file = None
+        if _has_option('cert_file'):
+            cert_file = _config_value('cert_file')
+
+        cert_key_file = None
+        if _has_option('cert_key_file'):
+            cert_key_file = _config_value('cert_key_file')
+
+        cert_key_password = None
+        if _has_option('cert_key_password'):
+            cert_key_password = _config_value('cert_key_password')
 
         connection_pool_maxsize = None
-        if config.has_option('influx2', 'connection_pool_maxsize'):
-            connection_pool_maxsize = config_value('connection_pool_maxsize')
+        if _has_option('connection_pool_maxsize'):
+            connection_pool_maxsize = _config_value('connection_pool_maxsize')
 
         auth_basic = False
-        if config.has_option('influx2', 'auth_basic'):
-            auth_basic = config_value('auth_basic')
+        if _has_option('auth_basic'):
+            auth_basic = _config_value('auth_basic')
 
         default_tags = None
-        if config.has_section('tags'):
-            tags = {k: v.strip('"') for k, v in config.items('tags')}
-            default_tags = dict(tags)
+        if _has_section('tags'):
+            if is_json:
+                default_tags = config['tags']
+            else:
+                tags = {k: v.strip('"') for k, v in config.items('tags')}
+                default_tags = dict(tags)
 
         profilers = None
-        if config.has_option('influx2', 'profilers'):
-            profilers = [x.strip() for x in config_value('profilers').split(',')]
+        if _has_option('profilers'):
+            profilers = [x.strip() for x in _config_value('profilers').split(',')]
 
         proxy = None
-        if config.has_option('influx2', 'proxy'):
-            proxy = config_value('proxy')
+        if _has_option('proxy'):
+            proxy = _config_value('proxy')
 
         return cls(url, token, debug=debug, timeout=_to_int(timeout), org=org, default_tags=default_tags,
                    enable_gzip=enable_gzip, verify_ssl=_to_bool(verify_ssl), ssl_ca_cert=ssl_ca_cert,
+                   cert_file=cert_file, cert_key_file=cert_key_file, cert_key_password=cert_key_password,
                    connection_pool_maxsize=_to_int(connection_pool_maxsize), auth_basic=_to_bool(auth_basic),
-                   profilers=profilers, proxy=proxy)
+                   profilers=profilers, proxy=proxy, **kwargs)
 
     @classmethod
-    def _from_env_properties(cls, debug=None, enable_gzip=False):
+    def _from_env_properties(cls, debug=None, enable_gzip=False, **kwargs):
         url = os.getenv('INFLUXDB_V2_URL', "http://localhost:8086")
         token = os.getenv('INFLUXDB_V2_TOKEN', "my-token")
         timeout = os.getenv('INFLUXDB_V2_TIMEOUT', "10000")
         org = os.getenv('INFLUXDB_V2_ORG', "my-org")
         verify_ssl = os.getenv('INFLUXDB_V2_VERIFY_SSL', "True")
         ssl_ca_cert = os.getenv('INFLUXDB_V2_SSL_CA_CERT', None)
+        cert_file = os.getenv('INFLUXDB_V2_CERT_FILE', None)
+        cert_key_file = os.getenv('INFLUXDB_V2_CERT_KEY_FILE', None)
+        cert_key_password = os.getenv('INFLUXDB_V2_CERT_KEY_PASSWORD', None)
         connection_pool_maxsize = os.getenv('INFLUXDB_V2_CONNECTION_POOL_MAXSIZE', None)
         auth_basic = os.getenv('INFLUXDB_V2_AUTH_BASIC', "False")
 
@@ -151,8 +215,9 @@ class _BaseClient(object):
 
         return cls(url, token, debug=debug, timeout=_to_int(timeout), org=org, default_tags=default_tags,
                    enable_gzip=enable_gzip, verify_ssl=_to_bool(verify_ssl), ssl_ca_cert=ssl_ca_cert,
+                   cert_file=cert_file, cert_key_file=cert_key_file, cert_key_password=cert_key_password,
                    connection_pool_maxsize=_to_int(connection_pool_maxsize), auth_basic=_to_bool(auth_basic),
-                   profilers=profilers)
+                   profilers=profilers, **kwargs)
 
 
 # noinspection PyMethodMayBeStatic
@@ -169,9 +234,9 @@ class _BaseQueryApi(object):
     """Base implementation for Queryable API."""
 
     def _to_tables(self, response, query_options=None, response_metadata_mode:
-                   FluxResponseMetadataMode = FluxResponseMetadataMode.full) -> List[FluxTable]:
+                   FluxResponseMetadataMode = FluxResponseMetadataMode.full) -> TableList:
         """
-        Parse HTTP response to FluxTables.
+        Parse HTTP response to TableList.
 
         :param response: HTTP response from an HTTP client. Expected type: `urllib3.response.HTTPResponse`.
         """
@@ -180,9 +245,9 @@ class _BaseQueryApi(object):
         return _parser.table_list()
 
     async def _to_tables_async(self, response, query_options=None, response_metadata_mode:
-                               FluxResponseMetadataMode = FluxResponseMetadataMode.full) -> List[FluxTable]:
+                               FluxResponseMetadataMode = FluxResponseMetadataMode.full) -> TableList:
         """
-        Parse HTTP response to FluxTables.
+        Parse HTTP response to TableList.
 
         :param response: HTTP response from an HTTP client. Expected type: `aiohttp.client_reqrep.ClientResponse`.
         """
@@ -191,9 +256,9 @@ class _BaseQueryApi(object):
                 pass
             return parser.table_list()
 
-    def _to_csv(self, response: HTTPResponse) -> Iterator[List[str]]:
+    def _to_csv(self, response: HTTPResponse) -> CSVIterator:
         """Parse HTTP response to CSV."""
-        return csv.reader(codecs.iterdecode(response, _UTF_8_encoding))
+        return CSVIterator(response)
 
     def _to_flux_record_stream(self, response, query_options=None,
                                response_metadata_mode: FluxResponseMetadataMode = FluxResponseMetadataMode.full) -> \
@@ -275,7 +340,7 @@ class _BaseQueryApi(object):
             from influxdb_client.client.query_api import QueryOptions
             return QueryOptions(profilers=self._influxdb_client.profilers)
 
-    def _create_query(self, query, dialect=default_dialect, params: dict = None):
+    def _create_query(self, query, dialect=default_dialect, params: dict = None, **kwargs):
         query_options = self._get_query_options()
         profilers = query_options.profilers if query_options is not None else None
         q = Query(query=query, dialect=dialect, extern=_BaseQueryApi._build_flux_ast(params, profilers))
@@ -286,6 +351,9 @@ class _BaseQueryApi(object):
             print("===============")
             print(query)
 
+        if kwargs.get('dataframe_query', False):
+            MissingPivotFunction.print_warning(query)
+
         return q
 
     @staticmethod
@@ -314,7 +382,9 @@ class _BaseQueryApi(object):
             return FloatLiteral("FloatLiteral", value)
         elif isinstance(value, datetime):
             value = get_date_helper().to_utc(value)
-            return DateTimeLiteral("DateTimeLiteral", value.strftime('%Y-%m-%dT%H:%M:%S.%fZ'))
+            nanoseconds = getattr(value, 'nanosecond', 0)
+            fraction = f'{(value.microsecond * 1000 + nanoseconds):09d}'
+            return DateTimeLiteral("DateTimeLiteral", value.strftime('%Y-%m-%dT%H:%M:%S.') + fraction + 'Z')
         elif isinstance(value, timedelta):
             _micro_delta = int(value / timedelta(microseconds=1))
             if _micro_delta < 0:
@@ -443,6 +513,8 @@ class _Configuration(Configuration):
     def __init__(self):
         Configuration.__init__(self)
         self.enable_gzip = False
+        self.username = None
+        self.password = None
 
     def update_request_header_params(self, path: str, params: dict):
         super().update_request_header_params(path, params)
