@@ -6,7 +6,7 @@ import logging
 import warnings
 
 from influxdb_client import HealthCheck, HealthService, Ready, ReadyService, PingService, \
-    InvocableScriptsApi
+    InvokableScriptsApi
 from influxdb_client.client._base import _BaseClient
 from influxdb_client.client.authorizations_api import AuthorizationsApi
 from influxdb_client.client.bucket_api import BucketsApi
@@ -24,13 +24,13 @@ logger = logging.getLogger('influxdb_client.client.influxdb_client')
 class InfluxDBClient(_BaseClient):
     """InfluxDBClient is client for InfluxDB v2."""
 
-    def __init__(self, url, token, debug=None, timeout=10_000, enable_gzip=False, org: str = None,
+    def __init__(self, url, token: str = None, debug=None, timeout=10_000, enable_gzip=False, org: str = None,
                  default_tags: dict = None, **kwargs) -> None:
         """
         Initialize defaults.
 
         :param url: InfluxDB server API url (ex. http://localhost:8086).
-        :param token: auth token
+        :param token: ``token`` to authenticate to the InfluxDB API
         :param debug: enable verbose logging of http requests
         :param timeout: HTTP client timeout setting for a request specified in milliseconds.
                         If one number provided, it will be total request timeout.
@@ -40,6 +40,12 @@ class InfluxDBClient(_BaseClient):
         :param org: organization name (used as a default in Query, Write and Delete API)
         :key bool verify_ssl: Set this to false to skip verifying SSL certificate when calling API from https server.
         :key str ssl_ca_cert: Set this to customize the certificate file to verify the peer.
+        :key str cert_file: Path to the certificate that will be used for mTLS authentication.
+        :key str cert_key_file: Path to the file contains private key for mTLS certificate.
+        :key str cert_key_password: String or function which returns password for decrypting the mTLS private key.
+        :key ssl.SSLContext ssl_context: Specify a custom Python SSL Context for the TLS/ mTLS handshake.
+                                         Be aware that only delivered certificate/ key files or an SSL Context are
+                                         possible.
         :key str proxy: Set this to configure the http proxy to be used (ex. http://localhost:3128)
         :key str proxy_headers: A dictionary containing headers that will be sent to the proxy. Could be used for proxy
                                 authentication.
@@ -50,10 +56,12 @@ class InfluxDBClient(_BaseClient):
         :key bool auth_basic: Set this to true to enable basic authentication when talking to a InfluxDB 1.8.x that
                               does not use auth-enabled but is protected by a reverse proxy with basic authentication.
                               (defaults to false, don't set to true when talking to InfluxDB 2)
+        :key str username: ``username`` to authenticate via username and password credentials to the InfluxDB 2.x
+        :key str password: ``password`` to authenticate via username and password credentials to the InfluxDB 2.x
         :key list[str] profilers: list of enabled Flux profilers
         """
         super().__init__(url=url, token=token, debug=debug, timeout=timeout, enable_gzip=enable_gzip, org=org,
-                         default_tags=default_tags, **kwargs)
+                         default_tags=default_tags, http_client_logger="urllib3", **kwargs)
 
         from .._sync.api_client import ApiClient
         self.api_client = ApiClient(configuration=self.conf, header_name=self.auth_header_name,
@@ -75,13 +83,26 @@ class InfluxDBClient(_BaseClient):
         self.close()
 
     @classmethod
-    def from_config_file(cls, config_file: str = "config.ini", debug=None, enable_gzip=False):
+    def from_config_file(cls, config_file: str = "config.ini", debug=None, enable_gzip=False, **kwargs):
         """
         Configure client via configuration file. The configuration has to be under 'influx' section.
 
+        :param config_file: Path to configuration file
+        :param debug: Enable verbose logging of http requests
+        :param enable_gzip: Enable Gzip compression for http requests. Currently, only the "Write" and "Query" endpoints
+                            supports the Gzip compression.
+        :key str proxy_headers: A dictionary containing headers that will be sent to the proxy. Could be used for proxy
+                                authentication.
+        :key urllib3.util.retry.Retry retries: Set the default retry strategy that is used for all HTTP requests
+                                               except batching writes. As a default there is no one retry strategy.
+        :key ssl.SSLContext ssl_context: Specify a custom Python SSL Context for the TLS/ mTLS handshake.
+                                         Be aware that only delivered certificate/ key files or an SSL Context are
+                                         possible.
+
         The supported formats:
             - https://docs.python.org/3/library/configparser.html
             - https://toml.io/en/
+            - https://www.json.org/json-en.html
 
         Configuration options:
             - url
@@ -90,6 +111,9 @@ class InfluxDBClient(_BaseClient):
             - timeout,
             - verify_ssl
             - ssl_ca_cert
+            - cert_file
+            - cert_key_file
+            - cert_key_password
             - connection_pool_maxsize
             - auth_basic
             - profilers
@@ -130,15 +154,45 @@ class InfluxDBClient(_BaseClient):
                 customer = "California Miner"
                 data_center = "${env.data_center}"
 
+        config.json example::
+
+            {
+                "url": "http://localhost:8086",
+                "token": "my-token",
+                "org": "my-org",
+                "active": true,
+                "timeout": 6000,
+                "connection_pool_maxsize": 55,
+                "auth_basic": false,
+                "profilers": "query, operator",
+                "tags": {
+                    "id": "132-987-655",
+                    "customer": "California Miner",
+                    "data_center": "${env.data_center}"
+                }
+            }
+
         """
         return super(InfluxDBClient, cls)._from_config_file(config_file=config_file, debug=debug,
-                                                            enable_gzip=enable_gzip)
+                                                            enable_gzip=enable_gzip, **kwargs)
 
     @classmethod
-    def from_env_properties(cls, debug=None, enable_gzip=False):
+    def from_env_properties(cls, debug=None, enable_gzip=False, **kwargs):
         """
         Configure client via environment properties.
 
+        :param debug: Enable verbose logging of http requests
+        :param enable_gzip: Enable Gzip compression for http requests. Currently, only the "Write" and "Query" endpoints
+                            supports the Gzip compression.
+        :key str proxy: Set this to configure the http proxy to be used (ex. http://localhost:3128)
+        :key str proxy_headers: A dictionary containing headers that will be sent to the proxy. Could be used for proxy
+                                authentication.
+        :key urllib3.util.retry.Retry retries: Set the default retry strategy that is used for all HTTP requests
+                                               except batching writes. As a default there is no one retry strategy.
+        :key ssl.SSLContext ssl_context: Specify a custom Python SSL Context for the TLS/ mTLS handshake.
+                                         Be aware that only delivered certificate/ key files or an SSL Context are
+                                         possible.
+
         Supported environment properties:
             - INFLUXDB_V2_URL
             - INFLUXDB_V2_ORG
@@ -146,12 +200,15 @@ class InfluxDBClient(_BaseClient):
             - INFLUXDB_V2_TIMEOUT
             - INFLUXDB_V2_VERIFY_SSL
             - INFLUXDB_V2_SSL_CA_CERT
+            - INFLUXDB_V2_CERT_FILE
+            - INFLUXDB_V2_CERT_KEY_FILE
+            - INFLUXDB_V2_CERT_KEY_PASSWORD
             - INFLUXDB_V2_CONNECTION_POOL_MAXSIZE
             - INFLUXDB_V2_AUTH_BASIC
             - INFLUXDB_V2_PROFILERS
             - INFLUXDB_V2_TAG
         """
-        return super(InfluxDBClient, cls)._from_env_properties(debug=debug, enable_gzip=enable_gzip)
+        return super(InfluxDBClient, cls)._from_env_properties(debug=debug, enable_gzip=enable_gzip, **kwargs)
 
     def write_api(self, write_options=WriteOptions(), point_settings=PointSettings(), **kwargs) -> WriteApi:
         """
@@ -245,13 +302,13 @@ class InfluxDBClient(_BaseClient):
         """
         return QueryApi(self, query_options)
 
-    def invocable_scripts_api(self) -> InvocableScriptsApi:
+    def invokable_scripts_api(self) -> InvokableScriptsApi:
         """
-        Create an InvocableScripts API instance.
+        Create an InvokableScripts API instance.
 
-        :return: InvocableScripts API instance
+        :return: InvokableScripts API instance
         """
-        return InvocableScriptsApi(self)
+        return InvokableScriptsApi(self)
 
     def close(self):
         """Shutdown the client."""
